package com.ibm.MQIsdp.midpapp;

import java.util.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import com.ibm.MQIsdp.*;
/**
 * This class contains a main entry point that simply prints out the version of the MQIsdp client
 * class and the version of the MQIsdp protocol that is supported in this implementation.
 */
public class AppMQIsdp extends MIDlet implements CommandListener, Runnable, MQIsdpAdvancedCallback {
    private ClientMQIsdp cm = null;
    private boolean running = true;
    private boolean connected = false;
    private boolean carryOn = false;
    private boolean deleteMsg = false;
    private Displayable currentDisplay = null;
    private Object lock = new Object();
    private Thread ourThread = null;
    private String clientId = null;
    private Vector inbox = new Vector();
    /**
     * Main entry point that simply prints out the version of the MQIsdp client
     * class and the version of the MQIsdp protocol that is supported in this implementation.
     */ 
    public AppMQIsdp() {
//        System.out.println( "ME  Enc:" + System.getProperty("microedition.encoding") );
//        System.out.println( "ME  Enc:" + System.getProperty("microedition.configuration") );
//        System.out.println( "ME  Enc:" + System.getProperty("microedition.profiles") );
        running = true;
    }

    public synchronized void destroyApp( boolean b ) {
        disconnect();
        running = false;
        ourThread = null;
    }
    public void pauseApp() {
    }
    public void startApp() {
        Thread ourThread = new Thread(this);
        ourThread.start();
    }
    
    public void run() {
        
        // Create a progress indicator display for use in the application
        Item[] progItems = new Item[1];
        progItems[0] = new Gauge("Executing...", false, Gauge.INDEFINITE, 0 );
        ((Gauge)progItems[0]).setValue( Gauge.CONTINUOUS_RUNNING );
//        progItems[0] = new Gauge("Executing...", false, 100, 0 );
//        ((Gauge)progItems[0]).setValue( 50 );
        Form progInd = new Form( "Progress Indicator", progItems );
        
        while ( running ) {
            while ( !connected) {
                // We are not being resumed, so request input data
                carryOn = false;
                // Display the connect panael then wait for input
                connectDisplay();
                try {
                    synchronized (lock) {
                        lock.wait();
                    }
                } catch(Exception e) {
                }
                if ( carryOn ) {
                    // Stick up a progress indicator, whilst we do the operation requested
                    Display.getDisplay(this).setCurrent( progInd );
                    
                    // Good input received, so carry on....
                    connect();
                } else {
                    // Exit application requested
                    running = false;
                    break;
                }
            }

            // Now allow publish, subscribe, unsubscribe or viewing of received publications
            // Set up the user interface objects before entering the
            // loop as we only want to create them once.
            String options[] = { "Publish", "Subscribe", "Unsubscribe", "Inbox" };
            List optsList = new List( "Options", List.IMPLICIT, options, null );
            optsList.setSelectCommand( new Command("Select", Command.ITEM, 1 ) );
//            optsList.addCommand( new Command("Select", Command.ITEM, 1 ) );
            optsList.addCommand( new Command("Cancel", Command.CANCEL, 1 ) );
            optsList.setCommandListener( this );

            carryOn = true;
            while ( connected ) {

                carryOn = false;
                // Display publish, subscribe or unsubscribe and wait for choice...
                Display.getDisplay(this).setCurrent( optsList );
                try {
                    synchronized (lock) {
                        lock.wait();
                    }
                } catch(Exception e) {
                }
                
                if ( carryOn == false ) {
                    // User has cancelled out of the pub, sub or unsub options panel, so disconnect
                    disconnect();
                }

                // Before entering the loop gathering input we can build a few objects that we continuously re-use
                // 1. Build a display as appropriate for the option selected.
                int selectedOption = optsList.getSelectedIndex();
                Displayable curOption = selectedOptionDisplay( selectedOption );

                while ( carryOn ) {
                    carryOn = false;
                    deleteMsg = false;
                    // Display the panel as created for publish, subscribe, unsubscribe or Inbox
                    Display.getDisplay(this).setCurrent( curOption );
                    try {
                        synchronized (lock) {
                            lock.wait();
                        }
                    } catch(Exception e) {
                    }
                    
                    if ( carryOn ) {
                        // If the Inbox was not selected then get the topic, which
                        // is the first field of the publish, subscribe and unsubscribe panels
                        Form f = null;
                        TextField topic = null;
                        String[] topics = null;
                        if ( selectedOption != 3 ) {
                            f = (Form)currentDisplay;

                            // Stick up a progress indicator, whilst we do the operation requested
                            Display.getDisplay(this).setCurrent( progInd );
                            
                            // Get the topic field
                            topic = (TextField)f.get(0);
                            topics = new String[1];
                        }

                        try {
                            switch ( selectedOption ) {
                            case 0:
                                // Publish
                                // Publish is non blocking, so wait for a published event from MQIsdp before continuing
                                TextField data = (TextField)f.get(1);
                                TextField qosData = (TextField)f.get(2);
                                int qos = Integer.parseInt( qosData.getString() );
                                if ( qos > 2 ) {
                                    qos = 2;
                                } else if ( qos < 0 ) {
                                    qos = 0;
                                }
                                cm.publish( topic.getString(),
                                            data.getString().getBytes(),
                                            qos,
                                            false );
                                // Wait until the publication has been delivered
                                // The published() callback method will notify the lock for publications with qos > 0
                                if ( qos > 0 ) {
                                    try {
                                        synchronized (lock) {
                                            lock.wait();
                                        }
                                    } catch(Exception e) {
                                    }
                                }

                                break;
                            case 1:
                                // Subscribe
                                // Subscribe is blocking, so continue once the API returns
                                int[] qosarr = new int[1];
                                topics[0] = topic.getString();
                                qosarr[0] = 1;
                                cm.subscribe( topics, qosarr );
                                break;
                            case 2:
                                // Unsubscribe
                                // Unsubscribe is blocking, so continue once the API returns
                                topics[0] = topic.getString();
                                cm.unsubscribe( topics );
                                break;
                             case 3:
                                 // Inbox
                                 // Display / Remove the message selected
                                 List l = (List)currentDisplay;
                                 if ( !deleteMsg ) {
                                     // Display message and wait for user interaction
                                     readMessageDisplay( (MQIsdpMsg)inbox.elementAt( l.getSelectedIndex() ) );
                                     try {
                                         synchronized (lock) {
                                             lock.wait();
                                         }
                                     } catch(Exception e) {
                                     }
                                 }

                                 if ( deleteMsg ) {
                                     // Delete the selected element and rebuild the inbox
                                     inbox.removeElementAt( l.getSelectedIndex() );
                                     curOption = selectedOptionDisplay( selectedOption );
                                 }
                                 break;
                            }
                            // Put in a short sleep otherwise the speed of display change
                            // ( e.g. publish -> progress indicator -> publish ) can happen too fast for the phone to handle
                            Thread.sleep(100);
                        } catch ( Exception e) {
                        }
                    }
                }
            }
        }

        // Clean ourselves up and notify the runtime that we are finished
        destroyApp(true);
        notifyDestroyed();
    }

    public void connectDisplay() {
        Item[] items = new Item[3];

        TextField t1 = new TextField( "Client Id", "MQISDP_MIDLET", 15, TextField.ANY );
        TextField t2 = new TextField( "IP address", "127.0.0.1", 15, TextField.ANY );
        TextField t3 = new TextField( "IP port", "1883", 4, TextField.NUMERIC );

        items[0] = (Item)t1;
        items[1] = (Item)t2;
        items[2] = (Item)t3;
        Form f = new Form( "Connection", items );

        Command c1 = new Command("Connect", Command.OK, 1 );
        Command c2 = new Command("Cancel", Command.CANCEL, 1 );

        f.addCommand( c1 );
        f.addCommand( c2 );
        f.setCommandListener( this );

        Display.getDisplay(this).setCurrent( f );
    }

    public Displayable selectedOptionDisplay( int selected ) {
        int numItems = 0;
        switch ( selected ) {
        case 0:
            // Publish
            numItems = 3;
            break;
        case 3:
            // Inbox
            return buildInbox();
        default:
            // Subscribe or Unsubscribe
            numItems = 1;
        }
        Item[] items = new Item[numItems];

        TextField t1 = new TextField( "Topic", null, 15, TextField.ANY );
        items[0] = (Item)t1;

        if ( selected == 0 ) {
            // Add publish fields
            TextField t2 = new TextField( "Data", null, 15, TextField.ANY );
            TextField t3 = new TextField( "QoS", "1", 1, TextField.NUMERIC );
            items[1] = (Item)t2;
            items[2] = (Item)t3;
        }

        String title = null;
        switch ( selected ) {
        case 0:
            title = "Publish";
            break;
        case 1:
            title = "Subscribe";
            break;
        case 2:
            title = "Unsubscribe";
            break;
        }
        
        Form f = new Form( title, items );

        f.addCommand( new Command( title, Command.OK, 1 ) );
        f.addCommand( new Command( "Cancel", Command.CANCEL, 1 ) );
        f.setCommandListener( this );

        return f;
    }
    
    public Displayable buildInbox() {

        if ( inbox.size() > 0 ) {
            String[] items = new String[inbox.size()];

            MQIsdpMsg m = null;
            int i = 0;
            for (Enumeration msgs = inbox.elements(); msgs.hasMoreElements() ; i++ ) {
                m = (MQIsdpMsg)msgs.nextElement();
                items[i] = m.getTopic();
            }
            
            List l = new List( "Topics received:", List.IMPLICIT, items , null );
            l.setSelectCommand( new Command( "Read", Command.ITEM, 1 ) );
//            l.addCommand( new Command( "Read", Command.ITEM, 1 ) );
            l.addCommand( new Command( "Delete", Command.ITEM, 2 ) );
            l.addCommand( new Command( "Cancel", Command.CANCEL, 3 ) );
            l.setCommandListener( this );

            return l;
        } else {
            Item[] items = new Item[1];
            items[0] = new StringItem( "Inbox empty", null );
            Form f = new Form( "Topics received:", items );
            f.addCommand( new Command( "Cancel", Command.CANCEL, 1 ) );
            f.setCommandListener( this );

            return f;
        }

    }
    
    public void readMessageDisplay( MQIsdpMsg theMsg ) {
        Item[] items = new Item[1];

        StringItem s1 = new StringItem( null, new String( theMsg.getData() ) );
        items[0] = (Item)s1;

        Form f = new Form( theMsg.getTopic(), items );

        f.addCommand( new Command("OK", Command.OK, 1 ) );
        f.addCommand( new Command("Delete", Command.CANCEL, 1 ) );
        f.setCommandListener( this );

        Display.getDisplay(this).setCurrent( f );
    }
    
    // Command action should not block otherwise it really screws the phone up !
    // Instead set some variables and notify the midlet thread that things have happened.
    public void commandAction( Command c, Displayable d ) {
        currentDisplay = d;
        carryOn = true;
        
        if ( c.getLabel().equals("Cancel") ) {
            // Cancel action
            carryOn = false;
        } else if ( c.getLabel().equals("Delete") ) {
            // Delete a message in the inbox.
            deleteMsg = true;
        }
        
        synchronized (lock) {
            lock.notify();
        }
    }

    // Implement the MQIsdpAdvancedCallback interface - published, publish Arrived and connectionLost
    // When a published event is received notify the application thread that it can coninue
    public void published( int msgId ) {
        synchronized (lock) {
            lock.notify();
        }
    }
    
    // If a reconnect fails throw an Exception and set the connected state to false.
    // This should return the user interface to the connection panel
    public void connectionLost() throws Exception {
        try {
            cm.connect( clientId, false, (short)15 );
        } catch ( Exception e ) {
            connected = false;
            throw e;
        }
    }

    public void publishArrived( String topicName, byte[] data, int qos, boolean retained ) {
        MQIsdpMsg m = new MQIsdpMsg( topicName, data );

        inbox.addElement( m );
    }

    public void connect() {
        Form f = (Form)currentDisplay;
        TextField t1 = (TextField)f.get(0);
        TextField t2 = (TextField)f.get(1);
        TextField t3 = (TextField)f.get(2);
        clientId = t1.getString();
        try {
            cm = new ClientMQIsdp( t2.getString(), Integer.parseInt( t3.getString() ) );
            cm.registerAdvancedHandler( this );
            cm.connect( clientId, false, (short)15 );
            connected = true;
        } catch ( RuntimeException re ) {
            // Throw runtime exceptions
            throw re;
        } catch (Exception e) {
            System.out.println( "Connect exception caught" );
            connected = false;
        }
    }

    public void disconnect() {
        try {
            if ( (cm != null) && connected ) {
                cm.disconnect();
                cm.terminate();
                cm = null;
            }
        } catch (Exception e) {
            System.out.println( "Disconnect exception caught" );
        }
        connected = false;
    }

}

class MQIsdpMsg {
    private String topic;
    private byte[] data;

    public MQIsdpMsg( String thisTopic, byte[] thisData ) {
        topic = thisTopic;
        data = thisData;
    }

    public String getTopic() {
        return topic;
    }

    public byte[] getData() {
        return data;
    }
}

