//-----------------------------------------------------------------------------
// Source File Name: ConnOpts.java
//
// Description: MQIsdp connect options dialog box
//              
// Licensed Materials - Property of IBM
//
// 5648-C63
// (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
//
// Version %Z% %W% %I% %E% %U%
//
//-----------------------------------------------------------------------------
package com.ibm.MQIsdp.utility;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

public class ConnOpts extends JDialog implements ActionListener {
	private static final String CLIENT_ID = "MQIsdp_Utility";
	private static final String KEEP_ALIVE = "30";
	private MQIsdpFrame mqisdpMgr = null;
	private JButton    trace;
    private JCheckBox  cleanSession;
    private JTextField clientId;
    private JTextField keepAlive;
    private JTextField lwtTopic;
    private JTextArea  lwtData;
    private JCheckBox  lwtRetain;
    private JComboBox  lwtQoS;
	private Integer[]  qos = { new Integer(0), new Integer(1), new Integer(2) };
    
	/**
	 * Constructor for ConnOpts.
	 */
	public ConnOpts( JFrame owner, String title, MQIsdpFrame mgr ) {
		super( owner, title );

        mqisdpMgr = mgr;
        
        // Get the container for this dialog and set the size and layout manager		
        Container connOptions = this.getContentPane();
        connOptions.setLayout( new BorderLayout() );
        defaultDialogSize( owner );

        // Create widgets to place on the GUI
        // Clean session check box
        cleanSession = new JCheckBox();
        cleanSession.setSelected(false);
        
        // Text field to hold the client id
        clientId = new JTextField( CLIENT_ID, 15 );
        clientId.setMaximumSize( MQIsdpFrame.TEXT_FIELD_DIMENSION );

        // Close and default buttons
        // Add actionlisteners and tooltips to the buttons
        JButton close = new JButton("Close");
        close.addActionListener(this);
        close.setToolTipText( "Options set will be saved" );
        
        JButton reset = new JButton("Default");
        reset.addActionListener(this);
        reset.setToolTipText( "Options will be set to default" );
        
        // Create components to go in the NORTH panel of the dialog
        // Top part
        JPanel top = new JPanel();
        top.setBorder( new EtchedBorder() );
        top.setLayout( new GridLayout(3,1) );

        // Create a title in BOLD with a trace button
        JLabel optsLabel = new JLabel("Session Options:    ", SwingConstants.LEFT);
        Font f = optsLabel.getFont();
        optsLabel.setFont( new Font( f.getName(), Font.BOLD, f.getSize() + 1 ) );
        trace = new JButton("Trace(Start)");
        trace.addActionListener( this );
        
        JPanel optsLabelAndTrace = new JPanel();
        optsLabelAndTrace.setLayout( new FlowLayout( FlowLayout.LEFT) );
        optsLabelAndTrace.add( optsLabel );
        optsLabelAndTrace.add( trace );
        

        // Create a Client Identifier field
        JPanel cid = new JPanel();
        cid.setLayout( new FlowLayout( FlowLayout.LEFT) );

        cid.add( new JLabel("Client Identifier:", SwingConstants.LEFT) );
        cid.add(clientId);
        
        // Create clean session and keep alive options
        JPanel clnSess = new JPanel();
        clnSess.setLayout( new FlowLayout( FlowLayout.LEFT ) );
        clnSess.add( new JLabel("Clean Session:", SwingConstants.LEFT) );
        clnSess.add( cleanSession );
        
        clnSess.add( new JLabel( "Keep Alive:", SwingConstants.LEFT ) );
        keepAlive = new JTextField(4);
        keepAlive.setText(KEEP_ALIVE);
        clnSess.add( keepAlive );
        clnSess.add( new JLabel( "seconds", SwingConstants.LEFT ) );
        
        // Now add all the components to the top part of the options panel
        top.add( optsLabelAndTrace );
        top.add( cid );
        top.add( clnSess );

        // Create components required for Last Will and Testament
        // CENTER pane - Last Will and Testament options
        // The CENTER pane of the dialog will contain a BorderLayout with the components laid out as follows:
        //       NORTH - title label, topic and options
        //       CENTER - LW&T message data
        JPanel middle = new JPanel();
        middle.setBorder( new EtchedBorder() );
        middle.setLayout( new BorderLayout() );

        // Create a LW&T topic field        
		lwtTopic = new JTextField( 15 );
        lwtTopic.setMaximumSize( MQIsdpFrame.TEXT_FIELD_DIMENSION );
        lwtTopic.setMargin( MQIsdpFrame.TEXT_MARGINS );
						
		// Create a text area for LW&T topic data				
		lwtData = new JTextArea(3,30);
		lwtData.setBorder( new BevelBorder( BevelBorder.LOWERED ) );
        lwtData.setMargin( MQIsdpFrame.TEXT_MARGINS );
						
		// Create a check box for LW&T retained topic option
        lwtRetain = new JCheckBox();
        lwtRetain.setSelected( false );
        
        // Create a dropdown box to hold the Last Will & Testament QoS options.
        lwtQoS = new JComboBox( qos );
        lwtQoS.setSelectedIndex( 0 );
		lwtQoS.setMaximumSize( MQIsdpFrame.DROP_DOWN_DIMENSION );

        // Create a title in BOLD
        JLabel lwtLabel = new JLabel("Last Will and Testament Settings:", SwingConstants.LEFT);
        Font fnt = lwtLabel.getFont();
        lwtLabel.setFont( new Font( fnt.getName(), Font.BOLD, fnt.getSize() + 1 ) );

        // Create a panel and add the LW&T topic and retained option to it
        JPanel lwtTopicOpts = new JPanel();
        lwtTopicOpts.setLayout( new BoxLayout( lwtTopicOpts, BoxLayout.X_AXIS) );

        lwtTopicOpts.add( new JLabel("Topic:") );
        lwtTopicOpts.add( lwtTopic );
        lwtTopicOpts.add( new JLabel(" QoS:") );
        lwtTopicOpts.add( lwtQoS );
        lwtTopicOpts.add( new JLabel(" Retained:") );
        lwtTopicOpts.add( lwtRetain );

        // Now add all the components for the LW&T NORTH panel to one panel
        JPanel lwtNorth = new JPanel();
        lwtNorth.setLayout( new GridLayout(2,1) );
        lwtNorth.add( lwtLabel );
        lwtNorth.add( lwtTopicOpts );
        
        // Finally construct the LW&T panel
        middle.add( lwtNorth, BorderLayout.NORTH );
        middle.add( new JScrollPane(lwtData), BorderLayout.CENTER );

        // Create button components
        // SOUTH pane - Close and reset buttons
        JPanel bottom = new JPanel();
        bottom.setBorder( new EtchedBorder() );
        bottom.add( close );
        bottom.add( reset );

        // Now put the complete connection options panel together
        connOptions.add(top, BorderLayout.NORTH );
        connOptions.add(middle, BorderLayout.CENTER );
        connOptions.add(bottom, BorderLayout.SOUTH );
        
        // Don't show the window until the options button is pressed
        this.setVisible( false );
	}
	
	public void defaultDialogSize( JFrame owner ) {
        this.setBounds( owner.getX() + 90,
                         owner.getY() + 50,
                         330, 305 );
	}	
	
	public boolean isCleanSessionSelected() {
		return cleanSession.isSelected();
	}	
	
	public String getClientID() {
		return clientId.getText();
	}	

    public short getKeepAlive() {
    	Integer i;
    	try {
    		i = new Integer( keepAlive.getText() );
    	} catch ( NumberFormatException e) {
    		// Invalid entry set - use the default
    		keepAlive.setText(KEEP_ALIVE);
    		return (short)30;
    	}		
    	return i.shortValue();
    }

	public boolean isLWTRetainSelected() {
		return lwtRetain.isSelected();
	}	

	public boolean isLWTTopicSet() {
		// See if the set text equals "" and return the inverse
		return !(lwtTopic.getText().equals(""));
	}	
	
	public String getLWTData() {
		return lwtData.getText();
	}	

    public String getLWTTopic() {
    	return lwtTopic.getText();
    }
    	
    public int getLWTQoS() {
    	return lwtQoS.getSelectedIndex();
    }

	public void actionPerformed(ActionEvent e) {
   		if ( e.getActionCommand().equals("Close") ) {   			
            this.setVisible(false);
   		} else if ( e.getActionCommand().equals("Trace(Start)") ) {    
   			try {
     			mqisdpMgr.startTrace();
     			trace.setText("Trace(Stop)");
   			} catch ( Exception ex ) {
   			}	
   		} else if ( e.getActionCommand().equals("Trace(Stop)") ) {    
   			mqisdpMgr.stopTrace();
   			trace.setText("Trace(Start)");
   		} else {
   			// Reset button has been pressed
   			// Reset all the fields to their default values
   			cleanSession.setSelected(false);
   			clientId.setText( CLIENT_ID );
   			keepAlive.setText(KEEP_ALIVE);
   			lwtTopic.setText("");
   			lwtData.setText("");
   			lwtRetain.setSelected(false);
   			lwtQoS.setSelectedIndex(0);
   		}	
	}	

}
